# -*- coding: utf-8 -*-

# ========================================
# >> IMPORTS
# ========================================

# Python
import random

# Python third-party
import configobj

# EventScripts
import es

# EventScripts libs
import cfglib
import gamethread
import playerlib
import popuplib
import weaponlib

# ========================================
# >> CONSTANTS
# ========================================

ADDON_PATH  = es.getAddonPath("seekme")
CURRENT_MAP = es.ServerVar("eventscripts_currentmap")

# ========================================
# >> GLOBAL VARIABLES
# ========================================

ccf_registered = False
map_globals    = None

# ========================================
# >> CLASSES
# ========================================

# provides easy reading access to a .cfg file
class Config(cfglib.AddonCFG):
    def __init__(self, path):
        super(Config, self).__init__(path)
        
        self.text("BackRaw's SeekMe v2.0")
        self.text("Configuration file")
        
        self.action   = self.cvar("seekme_action", "freeze", "Say whether to 'freeze' or 'blind' the seekers on round-start")
        self.knife    = self.cvar("seekme_knife", 1, "Give the hiders a knife each round for better chances to survive? 1/0")
        self.buyzone  = self.cvar("seekme_buyzone", 1, "1/0 for either enabling or disabling the buyzone for the hiders")
        self.duration = self.cvar("seekme_duration", 15, "How many seconds to apply the preferred actions and/or give the hiders a knife?")
    
    def __getattr__(self, attr):
        value = object.__getattribute__(self, attr)
        
        if attr in ("buyzon", "duration"):
            value = int(value)
        if attr == "knife":
            return bool(int(value))
        
        return value

config = Config(ADDON_PATH + "/seekme_config.cfg")

# gather per-map model infos every map-change
class MapGlobals(object):
    def __init__(self, path):
        self.models_config = configobj.ConfigObj(path)
        
        if not len(self.models_config.keys()):
            raise IOError, "Couldn't find file: %s" % path
        
        self.models = dict()
        self.popup  = popuplib.easymenu("_seekme_map_popup", "_popup_choice", self._popup_menu_select)
        
        self.popup.settitle("Choose a model for %s" % str(CURRENT_MAP))
        
        for model_basename in self.models_config.keys():
            model = self.models_config[model_basename]
            
            path = model["path"]
            name = model["name"]
            
            self.models[path] = name
            
            self.popup.addoption(path, name)
    
    def clean_up(self):
        self.models.clear()
        self.popup.delete()
    
    def _popup_menu_select(self, userid, choice, popupid):
        players[userid].set_info(choice)

# sub-classed dict for differentiating between model-specific and player-specific features
class Player(dict):
    def __init__(self, userid):
        super(Player, self).__init__()
        
        self["used"] = False
        self["name"] = None
        self["path"] = None
        
        self.stripped = False
        
        self.userid = userid
        self.player = playerlib.getPlayer(userid)
    
    # helper methods
    def tell(self, message):
        es.tell(self.userid, "#multi", "#green[#lightgreenSeekMe#green] #default%s" % message)
    
    def strip(self):
        if not self.stripped:
            weapon_list = list()
            
            if config.knife:
                arg = "#all,!knife"
            else:
                arg = "#all"
            
            for weapon in weaponlib.getWeaponList(arg):
                for index in weapon.indexlist:
                    if es.getindexprop(index, "CBaseEntity.m_hOwnerEntity") == self.player.handle:
                        es.server.queuecmd("es_xremove %s" % index)
                        break
            
            if config.knife:
                gamethread.queue(es.sexec, (self.userid, "use weapon_knife")) # just to be sure...
            
            self.stripped = True
            
            gamethread.queue(self.__setattr__, ("stripped", False))
            es.delayed(0, "es_xgive %s item_assaultsuit" % self.userid)
    
    def set_info(self, path):
        self["path"] = path
        self["name"] = map_globals.models[path]
        self["used"] = True
        
        self.strip()
        
        self.player.setModel(path)
        self.tell("Your current model: #green%s" % self["name"])
    
    # events
    def on_spawn(self):
        self.player.refreshAttributes()
        
        if not map_globals is None:
            if self.player.teamid == 2:
                # player is a hider
                
                self["used"] = False
                
                self.strip()
                
                if self.player.steamid == "BOT":
                    self.player.setModel(random.choice(map_globals.models.keys()))
                else:
                    if not self["path"] is None:
                        self.player.setModel(self["path"])
                        self.tell("Your current model: #green%s" % self["name"])
                    else:
                        map_globals.popup.send(self.userid)
                        self.tell("You can always use #green!hide #defaultto change your model to hide.")
            else:
                # player is a seeker
                
                if config.action == "freeze":
                    gamethread.delayed(2, self.player.freeze, 1)
                    gamethread.delayedname(config.duration, "_seekme_unfreeze_%s" % self.userid, self.player.freeze, 0)
                else:
                    self.player.flash(0, 0)
                    self.player.flash(255, config.duration)
    
    # say command
    def on_say_command_hide(self):
        self.player.refreshAttributes()
        
        if self.player.teamid == 2:
            if not self.player.isdead:
                if not self["used"]:
                    if not map_globals is None:
                        map_globals.popup.send(self.userid)
                    else:
                        self.tell("Config file for map #green%s #defaultisn't available yet." % str(CURRENT_MAP))
                else:
                    self.tell("You can only use #green!hide #defaultonce a round!")
            else:
                self.tell("Only alive Terrorists can change their models!")
        else:
            self.tell("CTs or Spectators can't change their models!")

# sub-classed dict which holds player information and provides safe removal
class Players(dict):
    def __init__(self):
        super(Players, self).__init__()
    
    def __getitem__(self, userid):
        return super(Players, self).__getitem__(int(userid))
    
    def __setitem__(self, userid, pm_object):
        super(Players, self).__setitem__(int(userid), pm_object)
    
    def clear(self):
        for userid in self.keys():
            self.remove(userid)
    
    def remove(self, userid):
        userid = int(userid)
        
        if userid in self.keys():
            temp = self[userid]
            
            gamethread.cancelDelayed("_seekme_unfreeze_%s" % userid)
            
            if temp.player.getFreeze():
                temp.player.freeze(0)
            
            if temp.player.getFlashAlpha():
                temp.player.flash(0, 0)
            
            temp.clear()
            
            del self[userid]

players = Players()

# ========================================
# >> HELPER FUNCTIONS
# ========================================

# since this piece of code is needed at load() and es_map_start(), I decided to put it in a helper function
def update_map_globals(map_name):
    global map_globals
    
    if not map_globals is None:
        map_globals.clean_up()
    
    del map_globals
    
    try:
        map_globals = MapGlobals(ADDON_PATH + "/models/%s.ini" % map_name)
    
    except IOError:
        map_globals = None

# ========================================
# >> EVENTS
# ========================================

def player_activate(ev):
    userid          = ev["userid"]
    players[userid] = Player(userid)

def player_spawn(ev):
    team = int(ev["es_userteam"])
    
    if team > 1:
        userid = int(ev["userid"])
        
        if userid in players.keys():
            players[userid].on_spawn()

# TODO: disable item-pickup with ease

def player_disconnect(ev):
    players.remove(ev["userid"])

def item_pickup(ev):
    player = players[ev["userid"]]
    
    if player.player.teamid == 2 and ev["item"].strip() != "knife":
        player.strip()

def es_map_start(ev):
    update_map_globals(ev["mapname"])

# ========================================
# >> SAY/CLIENT COMMANDS
# ========================================

def saycommand_hide():
    players[int(es.getcmduserid())].on_say_command_hide()

# ========================================
# >> SAY/CLIENT COMMAND FILTERS
# ========================================

# restrict per-Terrorist "buy" command if the buyzone option is disabled in the config file
def client_command_filter(userid, args):
    if args[0] == "buy":
        player = players[int(userid)]
        
        if player.player.teamid == 2:
            player.tell("Sorry, Terrorists are not allowed to buy (restricted by server admin).")
            return False
    
    return True

# ========================================
# >> LOAD & UNLOAD
# ========================================

def load():
    global ccf_registered
    
    # register public variable
    es.ServerVar("seekme_ver", "2.0").makepublic()
    
    # register say command !hide
    if not es.exists("saycommand", "!hide"):
        es.regsaycmd("!hide", "seekme/saycommand_hide")
    
    # add players to dictionary
    for userid in es.getUseridList():
        players[userid] = Player(userid)
    
    # update per-map models and popup
    update_map_globals(str(CURRENT_MAP))
    
    # write & execute the config file
    config.write()
    config.execute()
    
    # register client command filter if needed
    if not config.buyzone:
        es.addons.registerClientCommandFilter(client_command_filter)
        
        ccf_registered = True

def unload():
    # clean up
    players.clear()
    
    try:
        if not map_globals is None:
            map_globals.clean_up()
        # else, already cleaned up
    except NameError:
        pass # already cleaned up
    
    # un-register client command filter if it exists
    if ccf_registered:
        es.addons.unregisterClientCommandFilter(client_command_filter)
